@@ -28,8 +28,11 @@
 #include <KAboutData>
 #include <KCrash>
 #include <KLocalizedString>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <Kdelibs4ConfigMigrator>
 #include <Kdelibs4Migration>
+#endif
+#include <kconfigwidgets_version.h>
 #include <kdbusservice.h>
 
 using Konsole::Application;
@@ -74,29 +77,47 @@ public:
     }
 };
 
+// Used to control migrating config entries.
+// Increment when there are new keys to migrate.
+static int CurrentConfigVersion = 1;
+
 static void migrateRenamedConfigKeys()
 {
-    struct KeyInfo {
-        const char *groupName;
-        const char *oldKeyName;
-        const char *newKeyName;
-    };
-
-    static const KeyInfo keys[] = {{"KonsoleWindow", "SaveGeometryOnExit", "RememberWindowSize"}};
-
     KSharedConfigPtr konsoleConfig = KSharedConfig::openConfig(QStringLiteral("konsolerc"));
+    KConfigGroup verGroup = konsoleConfig->group("General");
+    const int savedVersion = verGroup.readEntry<int>("ConfigVersion", 0);
+    if (savedVersion < CurrentConfigVersion) {
+        struct KeyInfo {
+            const char *groupName;
+            const char *oldKeyName;
+            const char *newKeyName;
+        };
 
-    // Migrate renamed config keys
-    for (const auto &[group, oldName, newName] : keys) {
-        KConfigGroup cg = konsoleConfig->group(group);
-        if (cg.exists() && cg.hasKey(oldName)) {
-            const bool value = cg.readEntry(oldName, false);
-            cg.deleteEntry(oldName);
-            cg.writeEntry(newName, value);
+        static const KeyInfo keys[] = {{"KonsoleWindow", "SaveGeometryOnExit", "RememberWindowSize"}};
+
+        // Migrate renamed config keys
+        for (const auto &[group, oldName, newName] : keys) {
+            KConfigGroup cg = konsoleConfig->group(group);
+            if (cg.exists() && cg.hasKey(oldName)) {
+                const bool value = cg.readEntry(oldName, false);
+                cg.deleteEntry(oldName);
+                cg.writeEntry(newName, value);
+            }
         }
-    }
 
-    konsoleConfig->sync();
+        // With 5.93 KColorSchemeManager from KConfigWidgets, handles the loading
+        // and saving of the widget color scheme, and uses "ColorScheme" as the
+        // entry name, so clean-up here
+#if KCONFIGWIDGETS_VERSION >= QT_VERSION_CHECK(5, 93, 0)
+        KConfigGroup cg(konsoleConfig, "UiSettings");
+        const QString schemeName = cg.readEntry("WindowColorScheme");
+        cg.deleteEntry("WindowColorScheme");
+        cg.writeEntry("ColorScheme", schemeName);
+#endif
+
+        verGroup.writeEntry("ConfigVersion", CurrentConfigVersion);
+        konsoleConfig->sync();
+    }
 }
 
 // ***
@@ -108,11 +129,11 @@ int main(int argc, char *argv[])
     QElapsedTimer timer;
     timer.start();
 #endif
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     // enable high dpi support
     QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
     QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, true);
-
+#endif
     // Check if any of the arguments makes it impossible to re-use an existing process.
     // We need to do this manually and before creating a QApplication, because
     // QApplication takes/removes the Qt specific arguments that are incompatible.
@@ -126,17 +147,6 @@ int main(int argc, char *argv[])
 
     migrateRenamedConfigKeys();
 
-#if defined(Q_OS_MACOS)
-    // this ensures that Ctrl and Meta are not swapped, so CTRL-C and friends
-    // will work correctly in the terminal
-    app->setAttribute(Qt::AA_MacDontSwapCtrlAndMeta);
-
-    // KDE's menuBar()->isTopLevel() hasn't worked in a while.
-    // For now, put menus inside Konsole window; this also make
-    // the keyboard shortcut to show menus look reasonable.
-    app->setAttribute(Qt::AA_DontUseNativeMenuBar);
-#endif
-
     app->setWindowIcon(QIcon::fromTheme(QStringLiteral("utilities-terminal")));
 
     KLocalizedString::setApplicationDomain("konsole");
@@ -181,7 +191,7 @@ int main(int argc, char *argv[])
     KDBusService dbusService(startupOption | KDBusService::NoExitOnFailure);
 
     needToDeleteQApplication = false;
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     Kdelibs4ConfigMigrator migrate(QStringLiteral("konsole"));
     migrate.setConfigFiles({QStringLiteral("konsolerc"), QStringLiteral("konsole.notifyrc")});
     migrate.setUiFiles({QStringLiteral("sessionui.rc"), QStringLiteral("partui.rc"), QStringLiteral("konsoleui.rc")});
@@ -208,7 +218,7 @@ int main(int argc, char *argv[])
             }
         }
     }
-
+#endif
     // If we reach this location, there was no existing copy of Konsole
     // running, so create a new instance.
     Application konsoleApp(parser, customCommand);
@@ -260,6 +270,10 @@ bool shouldUseNewProcess(int argc, char *argv[])
         arguments.append(QString::fromLocal8Bit(argv[i]));
     }
 
+    if (arguments.contains(QLatin1String("--force-reuse"))) {
+        return false;
+    }
+
     // take Qt options into consideration
     QStringList qtProblematicOptions;
     qtProblematicOptions << QStringLiteral("--session") << QStringLiteral("--name") << QStringLiteral("--reverse") << QStringLiteral("--stylesheet")
