@@ -312,9 +312,11 @@ void TerminalPainter::drawContents(Character *image,
                 // What about the cursor?
                 // break;
             }
-            int log_x = vis2log(x);
+            const int log_x = vis2log(x);
+            const int log_next = vis2log(x + 1); // to know if this character is resolved as RTL, e.g. emojis in RTL context
 
             const Character char_value = image[pos + log_x];
+            const bool doubleWidth = image[qMin(pos + log_x + 1, imageSize - 1)].isRightHalfOfDoubleWide(); // East_Asian_Width wide character
 
             if (!printerFriendly && lastCharType == 0 && char_value.isSpace() && char_value.rendition.f.cursor == 0) {
                 continue;
@@ -327,8 +329,12 @@ void TerminalPainter::drawContents(Character *image,
 
             // paint text fragment
             if (unistr.length() && unistr[0] != QChar(0)) {
-                int textWidth = fontWidth * 1;
+                int textWidth = fontWidth * (doubleWidth ? 2 : 1);
                 int textX = leftPadding + fontWidth * x * (doubleWidthLine ? 2 : 1);
+                // East_Asian_Width wide character behaving as RTL, e.g. wide emoji inside RTL context
+                if (doubleWidth && log_next < log_x) {
+                    textX -= fontWidth * (doubleWidthLine ? 2 : 1);
+                }
                 if (!printerFriendly && char_value.rendition.f.cursor) {
                     Character style = char_value;
 
@@ -437,7 +443,6 @@ void TerminalPainter::drawContents(Character *image,
                           ulColorTable);
         }
 
-        paint.setRenderHint(QPainter::Antialiasing, false);
         paint.setWorldTransform(textScale.inverted(), true);
         if (lineProperty.flags.f.prompt_start && semanticHintsActive) {
             QPen pen(m_parentDisplay->terminalColor()->foregroundColor());
@@ -563,6 +568,19 @@ void TerminalPainter::drawBackground(QPainter &painter, const QRect &rect, const
     }
 }
 
+void TerminalPainter::updateCursorTextColor(const QColor &backgroundColor, QColor &characterColor)
+{
+    if (m_parentDisplay->cursorShape() == Enum::BlockCursor) {
+        if (m_parentDisplay->hasFocus()) {
+            // invert the color used to draw the text to ensure that the character at
+            // the cursor position is readable
+            QColor cursorTextColor = m_parentDisplay->terminalColor()->cursorTextColor();
+
+            characterColor = cursorTextColor.isValid() ? cursorTextColor : backgroundColor;
+        }
+    }
+}
+
 void TerminalPainter::drawCursor(QPainter &painter, const QRect &rect, const QColor &foregroundColor, const QColor &backgroundColor, QColor &characterColor)
 {
     if (m_parentDisplay->cursorBlinking()) {
@@ -590,11 +608,7 @@ void TerminalPainter::drawCursor(QPainter &painter, const QRect &rect, const QCo
         if (m_parentDisplay->hasFocus()) {
             painter.fillRect(cursorRect, cursorColor);
 
-            // invert the color used to draw the text to ensure that the character at
-            // the cursor position is readable
-            QColor cursorTextColor = m_parentDisplay->terminalColor()->cursorTextColor();
-
-            characterColor = cursorTextColor.isValid() ? cursorTextColor : backgroundColor;
+            updateCursorTextColor(backgroundColor, characterColor);
         } else {
             // draw the cursor outline, adjusting the area so that
             // it is drawn entirely inside cursorRect
@@ -711,12 +725,19 @@ void TerminalPainter::drawCharacters(QPainter &painter,
 
 void TerminalPainter::drawLineCharString(TerminalDisplay *display, QPainter &painter, int x, int y, const QString &str, const Character attributes)
 {
+    // only turn on anti-aliasing during this short time for the "text"
+    // for the normal text we have TextAntialiasing on demand on
+    // otherwise we have rendering artifacts
+    // set https://bugreports.qt.io/browse/QTBUG-66036
+    painter.setRenderHint(QPainter::Antialiasing, display->terminalFont()->antialiasText());
+
     const bool useBoldPen = (attributes.rendition.f.bold) != 0 && display->terminalFont()->boldIntense();
     QRect cellRect = {x, y, display->terminalFont()->fontWidth(), display->terminalFont()->fontHeight()};
     QVector<uint> ucs4str = str.toUcs4();
     for (int i = 0; i < ucs4str.length(); i++) {
         LineBlockCharacters::draw(painter, cellRect.translated(i * display->terminalFont()->fontWidth(), 0), ucs4str[i], useBoldPen);
     }
+    painter.setRenderHint(QPainter::Antialiasing, false);
 }
 
 void TerminalPainter::drawInputMethodPreeditString(QPainter &painter, const QRect &rect, TerminalDisplay::InputMethodData &inputMethodData, Character *image)
@@ -1050,8 +1071,7 @@ void TerminalPainter::drawTextCharacters(QPainter &painter,
         }
         characterColor = foregroundColor;
         if (style.rendition.f.cursor != 0) {
-            // Don't draw, only recalculate colors
-            drawCursor(painter, QRect(0, 0, 0, 0), foregroundColor, backgroundColor, characterColor);
+            updateCursorTextColor(backgroundColor, characterColor);
         }
         if (m_parentDisplay->filterChain()->showUrlHint()) {
             if ((style.flags & EF_REPL) == EF_REPL_PROMPT) {
